# Autogenerated by nbdev

d = { 'settings': { 'branch': 'main',
                'doc_baseurl': '/nymstr',
                'doc_host': 'https://dsm-72.github.io',
                'git_url': 'https://github.com/dsm-72/nymstr',
                'lib_path': 'nymstr'},
  'syms': { 'nymstr.core': { 'nymstr.core.nymstr': ('core.html#nymstr', 'nymstr/core.py'),
                             'nymstr.core.nymstr.__eq__': ('core.html#nymstr.__eq__', 'nymstr/core.py'),
                             'nymstr.core.nymstr.__new__': ('core.html#nymstr.__new__', 'nymstr/core.py'),
                             'nymstr.core.nymstr.__repr__': ('core.html#nymstr.__repr__', 'nymstr/core.py'),
                             'nymstr.core.nymstr.copy': ('core.html#nymstr.copy', 'nymstr/core.py'),
                             'nymstr.core.nymstr.isitem': ('core.html#nymstr.isitem', 'nymstr/core.py'),
                             'nymstr.core.nymstr.item': ('core.html#nymstr.item', 'nymstr/core.py'),
                             'nymstr.core.nymstr.snake': ('core.html#nymstr.snake', 'nymstr/core.py')},
            'nymstr.snake': { 'nymstr.snake.joinall': ('snake.html#joinall', 'nymstr/snake.py'),
                              'nymstr.snake.py_to_snake': ('snake.html#py_to_snake', 'nymstr/snake.py'),
                              'nymstr.snake.swapall': ('snake.html#swapall', 'nymstr/snake.py'),
                              'nymstr.snake.to_snake': ('snake.html#to_snake', 'nymstr/snake.py')}}}



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/01_snake.ipynb.

# %% auto 0
__all__ = ['BASIC_WORDS_TO_SNAKE', 'WORDS_TO_SNAKE_WITH_UPPERCASE', 'CAMEL_TO_SNAKE', 'PASCAL_TO_SNAKE', 'SUNDER', 'DUNDER',
           'SUB_SUNDER', 'SNAKE_PATTERNS', 'swapall', 'joinall', 'to_snake', 'py_to_snake']

# %% ../nbs/01_snake.ipynb 4
import re

# %% ../nbs/01_snake.ipynb 8
BASIC_WORDS_TO_SNAKE = re.compile(
    r'[A-Z]?[a-z]+'                     # A possible uppercase followed by lowercase letters
    r'|[A-Z]{2,}(?=[A-Z][a-z]|\d|\W|$)' # Two or more consecutive uppercase letters
    r'|\d+'                             # One or more digits
)

# %% ../nbs/01_snake.ipynb 9
WORDS_TO_SNAKE_WITH_UPPERCASE = re.compile(
    r'[A-Z]?[a-z]+'                     # A possible uppercase followed by lowercase letters
    r'|[A-Z]{2,}(?=[A-Z][a-z]|\d|\W|$)' # Two or more consecutive uppercase letters
    r'|\d+'                             # One or more digits
    r'|[A-Z]{2,}'                       # Two or more consecutive uppercase letters
    r'|[A-Z]$'                          # Uppercase letter at end of string
)

# %% ../nbs/01_snake.ipynb 13
CAMEL_TO_SNAKE = re.compile(
    r'(?<=[a-z0-9])[A-Z]'          # An uppercase letter after a lowercase letter / digit
    r'|(?<![A-Z_])[A-Z](?=[a-z])'  # An uppercase letter before a lowercase letter AND 
                                   # not after an uppercase letter or underscore
)

# %% ../nbs/01_snake.ipynb 17
PASCAL_TO_SNAKE = re.compile(
    r'(?!^)(?<!_)[A-Z][a-z]+'     # An uppercase letter not at the start and not after an underscore, 
                                  # followed by one or more lowercase letters (captures beginning of PascalCase words)
    r'|(?<=[a-z0-9])[A-Z]'        # An uppercase letter after a lowercase letter or digit 
                                  # (captures transition in camelCase)
)

# %% ../nbs/01_snake.ipynb 20
SUNDER = '_'
DUNDER = '__'

SUB_SUNDER = rf'{SUNDER}\1'

# %% ../nbs/01_snake.ipynb 21
def swapall(s: str, pattern: str, repl: str = SUB_SUNDER) -> str:
    '''Replace all occurences of the pattern in string with repl'''
    return re.sub(pattern, repl, s)

def joinall(s: str, pattern: str, repl: str = SUNDER) -> str:
    '''Join all occurrences of the pattern in string with repl'''
    matches = re.findall(pattern, s)
    return repl.join(matches)

# %% ../nbs/01_snake.ipynb 22
def to_snake(s: str) -> str:
    '''Convert a string to snake_case'''
    return joinall(s, WORDS_TO_SNAKE_WITH_UPPERCASE, SUNDER).lower()
    
def py_to_snake(s: str) -> str:    
    _precap = [f'_{c.lower()}' if c.isupper() else c for c in s]
    return ''.join(_precap).lstrip(SUNDER)

# %% ../nbs/01_snake.ipynb 23
SNAKE_PATTERNS = (BASIC_WORDS_TO_SNAKE, WORDS_TO_SNAKE_WITH_UPPERCASE, CAMEL_TO_SNAKE, PASCAL_TO_SNAKE,)



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/00_core.ipynb.

# %% auto 0
__all__ = ['nymstr']

# %% ../nbs/00_core.ipynb 5
import copy

from atyp import AnyQ, StrLikeQ, ClsStrQ, ClsBool, ClsFloat

from indoc import indoc
from fuzstr import fuzstr
from dynattr import DynamicAttributesMixin

# from aprep import aprep

# %% ../nbs/00_core.ipynb 6
from ispec.enums import (AttrPref,)
from ispec.utils import (funkws, kwsopt, )
from ispec.dun import (getnew, getsup, )
from ispec.agg import aggattrs
from ispec.set import setkws

# %% ../nbs/00_core.ipynb 7
from .snake import to_snake

# %% ../nbs/00_core.ipynb 9
@indoc
class nymstr(fuzstr, DynamicAttributesMixin):    
    '''nymstr

    Parameters
    ----------    
    itemattr : str, Default: 'el'
        The attribute which holds the item being named, by default 'el'.
    fullrepr : bool, Default: False
        Whether or not to show the full representation, by default False.
    '''
    # Whether to ignore case when matching
    icase:  ClsBool  = False
    # Whether to ignore spaces when matching
    ispace: ClsBool  = False
    # Whether to ignore underscores when matching
    iunder: ClsBool  = False
    # The cutoff for fuzzy matching
    cutoff: ClsFloat = 1

    itemattr: ClsStrQ = 'el'
    fullrepr: ClsBool = False

    def __new__(cls, name: StrLikeQ, el: AnyQ = None, **kwargs):        
        # NOTE: `aggattr` gets all kws, whereas `cls.prepall(cls)` only gets annotated kws
        kws, supnew= aggattrs(cls, -1), getnew(getsup(cls))
        kws = kwsopt(kws, **kwargs)        
        # Filter for super().__new__ parameters
        supkws = funkws(supnew, **kws) 
        # Create the original "str" (fuzstr)
        obj = super().__new__(cls, name, **supkws)

        # Extract the name and item from the kwargs  
        obj = setkws(obj, pref=AttrPref.KOC, keys=list(kws.keys()), **kws) 

        # Set dynamic attributes
        dattrs = ('itemattr', )
        kws.update({cls.itemattr: el})
        obj.setdynattrs(dattrs, **kws)
        return obj

    def __repr__(self):
        if self.getclsattr('fullrepr') or self.fullrepr:
            return f'{self.clsname}({str(self)}={self.item})'
        return str(self)

    @property
    def snake(self) -> str:
        return to_snake(self)

    def copy(self):
        return copy.copy(self)

    @property
    def item(self) -> str:
        '''The dynamic `itemattr` i.e. `getattr(self, self.itemattr, None)`'''
        return self.getdynattr('itemattr', None)        
    
    def isitem(self, other: AnyQ) -> bool:
        '''Check if `other` is the instance's item'''        
        if isinstance(other, type(self)) or hasattr(other, 'item'):
            return self.item == other.item
        return self.item == other

    def __eq__(self, other):
        if isinstance(other, (type(self))):
            return super().__eq__(other) and self.isitem(other)
        return super().__eq__(other)
